import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { useCallback, useContext, useEffect, useState } from 'react';
import { ErrorContext } from '../hooks/error';
import { useRequest, useSSE } from '../tools';
import { DefaultSpinner, DefaultNotFound, DefaultTransition } from '../Defaults';
import { ConfigContext } from '../hooks/config';
import { usePageEventListen } from '../events';
import { EventContextProvider, useEventContext } from '../hooks/eventContext';
import { LocationContext } from '../hooks/locationContext';
import { AnyCompList } from './index';
export const ServerLoadComp = ({ path, components, loadTrigger, sse }) => {
    if (components) {
        return _jsx(ServerLoadDefer, { path: path, components: components, loadTrigger: loadTrigger, sse: sse });
    }
    else if (sse) {
        return _jsx(ServerLoadSSE, { path: path });
    }
    else {
        return _jsx(ServerLoadFetch, { path: path });
    }
};
const ServerLoadDefer = ({ components, path, loadTrigger, sse, }) => {
    const { eventContext } = usePageEventListen(loadTrigger);
    if (eventContext) {
        return (_jsx(EventContextProvider, { context: eventContext, children: sse ? _jsx(ServerLoadSSE, { path: path }) : _jsx(ServerLoadFetch, { path: path }) }));
    }
    else {
        return _jsx(AnyCompList, { propsList: components });
    }
};
export const ServerLoadFetch = ({ path, devReload }) => {
    const [transitioning, setTransitioning] = useState(false);
    const [componentProps, setComponentProps] = useState(null);
    const [notFoundUrl, setNotFoundUrl] = useState(undefined);
    const { fullPath } = useContext(LocationContext);
    const url = useServerUrl(path);
    const request = useRequest();
    useEffect(() => {
        setTransitioning(true);
        const promise = request({ url, expectedStatus: [200, 404] });
        promise.then(([status, data]) => {
            if (status === 200) {
                setComponentProps(data);
                // if there's a fragment, scroll to that ID once the page is loaded
                const fragment = getFragment(path);
                if (fragment) {
                    setTimeout(() => {
                        const element = document.getElementById(fragment);
                        if (element) {
                            element.scrollIntoView();
                        }
                    }, 50);
                }
            }
            else {
                setNotFoundUrl(url);
            }
            setTransitioning(false);
        });
        return () => {
            promise.then(() => null);
        };
    }, [url, path, request, devReload]);
    useEffect(() => {
        setNotFoundUrl(undefined);
    }, [fullPath]);
    return _jsx(Render, { propsList: componentProps, notFoundUrl: notFoundUrl, transitioning: transitioning });
};
export const ServerLoadSSE = ({ path }) => {
    const [componentProps, setComponentProps] = useState(null);
    const url = useServerUrl(path);
    const onMessage = useCallback((data) => setComponentProps(data), []);
    useSSE(url, onMessage);
    return _jsx(Render, { propsList: componentProps, transitioning: false });
};
const Render = ({ propsList, notFoundUrl, transitioning, }) => {
    const { error } = useContext(ErrorContext);
    const { Spinner, NotFound, Transition } = useContext(ConfigContext);
    const SpinnerComp = Spinner ?? DefaultSpinner;
    const NotFoundComp = NotFound ?? DefaultNotFound;
    const TransitionComp = Transition ?? DefaultTransition;
    if (notFoundUrl) {
        return _jsx(NotFoundComp, { url: notFoundUrl });
    }
    else if (propsList === null) {
        if (error) {
            return _jsx(_Fragment, {});
        }
        else {
            return _jsx(SpinnerComp, {});
        }
    }
    else {
        return (_jsx(TransitionComp, { transitioning: transitioning, children: _jsx(AnyCompList, { propsList: propsList }) }));
    }
};
function useServerUrl(path) {
    const { rootUrl, pathSendMode } = useContext(ConfigContext);
    const applyContext = useEventContext();
    const requestPath = applyContext(path);
    if (pathSendMode === 'query') {
        return `${rootUrl}?path=${encodeURIComponent(requestPath)}`;
    }
    else {
        return rootUrl + requestPath;
    }
}
function getFragment(path) {
    const index = path.indexOf('#');
    if (index !== -1) {
        return path.slice(index + 1);
    }
}
