import { useCallback, useContext, useEffect } from 'react';
import { ErrorContext } from './hooks/error';
export function useRequest() {
    const { setError } = useContext(ErrorContext);
    return useCallback(async (args) => {
        try {
            return await request(args);
        }
        catch (e) {
            setError({ title: 'Request Error', description: e?.message });
            throw e;
        }
    }, [setError]);
}
export function useSSE(url, onMessage) {
    useEffect(() => {
        const source = new EventSource(url);
        source.onmessage = (e) => {
            const data = JSON.parse(e.data);
            onMessage(data);
        };
        source.onerror = (e) => {
            // we don't raise an error her as this can happen when the server is restarted
            console.debug('SSE error', e);
        };
        const cleanup = () => {
            source.onerror = null;
            source.close();
        };
        window.addEventListener('beforeunload', cleanup);
        return () => {
            window.removeEventListener('beforeunload', cleanup);
            cleanup();
        };
    }, [url, onMessage]);
}
class RequestError extends Error {
    constructor(message, status) {
        super(message);
        Object.defineProperty(this, "status", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.status = status;
        this.name = 'RequestError';
    }
}
async function request({ url, method, headers, query, json, expectedStatus, formData, }) {
    const init = {};
    let contentType = null;
    if (json) {
        init.body = JSON.stringify(json);
        contentType = 'application/json';
        method = method ?? 'POST';
    }
    else if (formData) {
        // don't set content-type, let the browser set it
        init.body = formData;
        method = method ?? 'POST';
    }
    if (query) {
        const searchParams = new URLSearchParams(query);
        url = `${url}?${searchParams.toString()}`;
    }
    headers = headers ?? {};
    if (contentType && !headers['Content-Type']) {
        headers['Content-Type'] = contentType;
    }
    if (method) {
        init.method = method;
    }
    if (headers) {
        init.headers = headers;
    }
    let response;
    try {
        response = await fetch(url, init);
    }
    catch (e) {
        throw new RequestError('fetch failed', 0);
    }
    const { status } = response;
    if (!responseOk(response, expectedStatus)) {
        let detail = null;
        const content = await response.text();
        try {
            const jsonData = JSON.parse(content);
            console.warn(`${url} -> ${status} JSON:`, jsonData);
            if (typeof jsonData.detail === 'string') {
                detail = jsonData.detail;
            }
        }
        catch (e) {
            console.warn(`${url} -> ${status} content:`, content);
            detail = content;
        }
        const msg = `${detail || response.statusText} (${status})`;
        throw new RequestError(msg, status);
    }
    let data;
    try {
        data = await response.json();
    }
    catch (e) {
        console.warn(`${url} -> ${status} response not valid JSON`);
        throw new RequestError('Response not valid JSON', status);
    }
    console.debug(`${url} -> ${status} JSON:`, data);
    return [status, data];
}
function responseOk(response, expectedStatus) {
    if (expectedStatus) {
        return expectedStatus.includes(response.status);
    }
    else {
        return response.ok;
    }
}
export function unreachable(msg, unexpectedValue, args) {
    console.warn(msg, { unexpectedValue }, args);
}
export function debounce(fn, delay) {
    let timerId;
    // @ts-expect-error - functions are contravariant, so this should be fine, no idea how to satisfy TS though
    return (...args) => {
        clearTimeout(timerId);
        timerId = setTimeout(() => fn(...args), delay);
    };
}
export async function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
// usage `as_title('what_ever') > 'What Ever'`
export const asTitle = (s) => s.replace(/[_-]/g, ' ').replace(/(_|\b)\w/g, (l) => l.toUpperCase());
export const slugify = (s) => s
    .toLowerCase()
    .replace(/\s+/g, '-') // Replace spaces with -
    .replace(/[^\w-]+/g, '') // Remove all non-word characters
    .replace(/--+/g, '-') // Replace multiple - with single -
    .replace(/^-+/, '') // Trim - from start of text
    .replace(/-+$/, ''); // Trim - from end of text
